# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2013, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import tools
from openerp.osv import fields
from openerp.osv.orm import Model
from openerp.tools import float_round
import openerp.addons.decimal_precision as dp


class flight_order(Model):
    _name = 'flight.order'
    _inherit = ['mail.thread']
    _description = 'Flight Order'

    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        for order in self.browse(cr, uid, ids, context=context):
            res[order.id] = {
                'amount_total': 0,
                'commission_total': 0.0,
            }
            val = 0
            val1 = 0.0
            for line in order.line_ids:
                val1 += line.commission * line.quantity
                val += 1 * line.quantity
            res[order.id]['amount_total'] = val
            res[order.id]['commission_total'] = val1
        return res

    _columns = {
        'name': fields.char('Name', size=64, select=True, readonly=True, states={'draft': [('readonly', False)]}),
        'comment': fields.text('Additional Information', states={'draft': [('readonly', False),]}),
        'date_order': fields.date('Date', required=True, readonly=True, select=True,
                                  states={'open': [('readonly', False),]}),
        'state': fields.selection([
            ('draft', 'Draft'),
            ('open', 'Open'),
            ('done', 'Confirmed'),
            ('cancel', 'Cancelled'),
            ], 'Status', select=True, readonly=True, track_visibility='onchange',
            help=' * The \'Draft\' status is used when a user is encoding a new and unconfirmed Flight. \
            \n* The \'Open\' status is used when user confirm the flight order report. \
            \n* The \'Cancelled\' status is used when user cancel flight order report.'),
        'period_id': fields.many2one('account.period', 'flight Period', help="Select flight period to reporting.",
                                     readonly=True, states={'open': [('readonly', False)]}),
        'line_ids': fields.one2many('flight.order.line', 'flight_order_id', 'flight Order Lines', readonly=True,
                                   states={'open': [('readonly', False)]}),
        'amount_total': fields.function(_amount_all, digits_compute=dp.get_precision('flight'), string='Total Flight',
            store={
                'flight.order': (lambda self, cr, uid, ids, c={}: ids, ['line_ids'], 10),
            },
            multi='sums', help="The total amount."),
        'commission_total': fields.function(_amount_all, digits_compute=dp.get_precision('flight'), string='Total Commission',
            store={
                'flight.order': (lambda self, cr, uid, ids, c={}: ids, ['line_ids'], 10),
            },
            multi='sums', help="The total commission."),


    }

    _defaults = {
        'state': 'draft',
        'date_order': fields.date.context_today,
    }

    def onchange_period(self, cr, uid, ids, period_id, line_ids, context=None):
        res = {'value': {}}
        period_obj = self.pool.get('account.period')
        sale_order_line_obj = self.pool.get('sale.order.line')
        # account_invoice_line_obj = self.pool.get('account.invoice.line')
        flight_order_line_obj = self.pool.get('flight.order.line')
        flight_supplier_obj = self.pool.get('flight.supplier')

        cr.execute('SELECT rel.product_id FROM product_flight AS rel')
        product_flight = cr.fetchall()
        cr.execute('SELECT rel.supplier_id FROM flight_supplier AS rel')
        flight_supplier = cr.fetchall()
        if period_id:
            period = period_obj.browse(cr, uid, period_id, context=context)
            # sale
            if flight_supplier:
                # order_sale = sale_order_line_obj.search(cr, uid, [('product_id', 'in', product_flight),
                #                                                   ('supplier_id', 'in', flight_supplier),
                #                                                   ('state', 'in', ['confirmed', 'done']),
                #                                                   ('order_start_date', '>=', period.date_start),
                #                                                   ('order_start_date', '<=', period.date_stop)],
                #                                         order='supplier_id ASC,customer_id ASC,name ASC',
                #                                         context=context)
                cr.execute("SELECT "
                            "  public.sale_order_line.id"
                            " FROM"
                            "  public.sale_order_line inner join public.sale_order ON (public.sale_order_line.order_id = public.sale_order.id)"
                            " WHERE sale_order_line.product_id IN %s AND"
                            "  sale_order_line.supplier_id IN %s AND"
                            "  sale_order_line.state IN ('confirmed','done') AND"
                            "  sale_order.date_confirm >= %s AND"
                            "  sale_order.date_confirm <= %s"
                            "  ORDER BY"
                            "  sale_order_line.supplier_id asc,"
                            "  sale_order_line.customer_id asc,"
                            " sale_order_line.name asc", (tuple(product_flight), tuple(flight_supplier), period.date_start, period.date_stop,))
                order_sale = cr.fetchall()
            else:
                # order_sale = sale_order_line_obj.search(cr, uid, [('product_id', 'in', product_flight),
                #                                                   ('state', 'in', ['confirmed', 'done']),
                #                                                   ('order_start_date', '>=', period.date_start),
                #                                                   ('order_start_date', '<=', period.date_stop)],
                #                                         order='supplier_id ASC,customer_id ASC,name ASC',
                #                                         context=context)
                cr.execute("SELECT "
                            "  public.sale_order_line.id"
                            " FROM"
                            "  public.sale_order_line inner join public.sale_order ON (public.sale_order_line.order_id = public.sale_order.id)"
                            " WHERE sale_order_line.product_id IN %s AND"
                            "  sale_order_line.state IN ('confirmed','done') AND"
                            "  sale_order.date_confirm >= %s AND"
                            "  sale_order.date_confirm <= %s"
                            "  ORDER BY"
                            "  sale_order_line.supplier_id asc,"
                            "  sale_order_line.customer_id asc,"
                            " sale_order_line.name asc", (tuple(product_flight), period.date_start, period.date_stop,))
                order_sale = cr.fetchall()

            for id in ids:
                flight_list = []
                obj = self.browse(cr, uid, id, context=context)
                if obj.line_ids:
                    black_list = []
                    for line in obj.line_ids:
                        black_list.append(line.id)
                    flight_order_line_obj.unlink(cr, uid, black_list, context=None)
                if order_sale:
                    list_sale = []
                    for x in order_sale:
                        list_sale.append(x[0])

                for sales in sale_order_line_obj.browse(cr, uid, list_sale, context=context):
                    flight_supplier_id = flight_supplier_obj.search(cr, uid,
                                                                    [('supplier_id', '=', sales.supplier_id.id)],
                                                                    order='supplier_id ASC', context=context)
                    if flight_supplier_id:
                        supplier_commission = flight_supplier_obj.browse(cr, uid, flight_supplier_id[0], context=context).commission

                    flight_order_line = {
                        'comment': sales.name,
                        'reservation_number': sales.reservation_number,
                        'customer_id': sales.customer_id.id,
                        'supplier_id': sales.supplier_id.id,
                        'quantity': sales.product_uom_qty,
                        'commission': supplier_commission or 10.0,
                        'flight_order_id': id
                    }

                    flight_id = flight_order_line_obj.create(cr, uid, flight_order_line, context=context)
                    flight_list.append(flight_id)

                res['value'] = {
                    'line_ids': flight_list
                }

        return res

    def action_cancel(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        self.write(cr, uid, ids, {'state': 'cancel'})
        return True

    def action_cancel_draft(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        self.write(cr, uid, ids, {'state': 'draft'})
        return True

    def action_button_open(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'open'})
        return True

    def action_button_confirm(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'done'})
        return True

    def button_dummy(self, cr, uid, ids, context=None):
        return True

    def create(self, cr, uid, vals, context=None):
        if vals.get('name','/')=='/':
            vals['name'] = self.pool.get('ir.sequence').get(cr, uid, 'flight.order') or '/'
        return super(flight_order, self).create(cr, uid, vals, context=context)


class flight_order_line(Model):
    _name = 'flight.order.line'
    _description = 'Flight Order Line'
    _order = 'flight_order_id desc, id'

    def _amount_line(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        for order in self.browse(cr, uid, ids, context=context):
            res[order.id] = order.commission * order.quantity
        return res

    _columns = {
        'flight_order_id': fields.many2one('flight.order', 'flight Order'),
        'customer_id': fields.many2one('res.partner', 'Costumer',
                                       domain="[('customer', '=', True)]",
                                       ),
        'supplier_id': fields.many2one('res.partner', 'Supplier',
                                       domain="[('supplier', '=', True)]",
                                       ),
        'comment': fields.text('Additional Information'),
        'reservation_number': fields.char('Reservation', size=64),
        'commission': fields.float('Commission x Pax',
                                   digits_compute=dp.get_precision('Commission'), required=True),
        'quantity': fields.float('Quantity', digits_compute=dp.get_precision('Product Unit of Measure'), required=True),
        'amount_subtotal': fields.function(_amount_line, string='Commission Subtotal', type="float",
                                           digits_compute=dp.get_precision('flight'), store=True),
    }

    _defaults = {
        'commission': 10.0,
        'quantity': 1,
    }
    #
    # def create(self, cr, uid, vals, context=None):
    #     if not context:
    #         context = {}
    #     return super(flight_order_line, self).create(cr, uid, vals, context=context)
    #
    # def write(self, cr, uid, vals, context=None):
    #     if not context:
    #         context = {}
    #     return super(flight_order_line, self).write(cr, uid, vals, context=context)