# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import tools
from openerp.osv import fields
from openerp.osv.orm import Model


class travel_product_purchase_invoice_report(Model):
    _name = "travel.product.purchase.invoice.report"
    _description = "Product Purchase Invoices Statistics"

    _auto = False
    _rec_name = 'year'

    _columns = {
        # 'partner_id': fields.many2one('res.partner', 'Supplier', readonly=True),
        'categ_id': fields.many2one('product.category', 'Category', readonly=True),
        # 'invoice_user': fields.many2one('res.users', 'Salesperson', readonly=True),
        'year': fields.char('Year', size=4, readonly=True),
        'mont_total': fields.float('Amount Invoiced', readonly=True),
        'mont_current': fields.float('Amount Actual Invoiced', readonly=True),
        'mont_primer_c': fields.float('Amount First Trimester', readonly=True),
        'mont_segundo_c': fields.float('Amount Second Trimester', readonly=True),
        'mont_tercer_c': fields.float('Amount Third Trimester', readonly=True),
        'mont_cuarto_c': fields.float('Amount Fourth Trimester', readonly=True),
    }
    _order = 'year desc,mont_total desc'

    def init(self, cr):
        tools.drop_view_if_exists(cr, 'travel_product_purchase_invoice_report')
        cr.execute("""
            create or replace view travel_product_purchase_invoice_report as (
                 WITH source AS (
                     SELECT c.product_id,
                        c.year,
                        c.primer_c,
                        c.segundo_c,
                        c.tercer_c,
                        c.cuarto_c,
                        sum(c.price_base_currency) OVER (PARTITION BY c.product_id, c.year) AS mont_total,
                        sum(c.amount_current) OVER (PARTITION BY c.product_id, c.year) AS mont_current,
                        sum(c.primer_c) OVER (PARTITION BY c.product_id, c.year) AS mont_primer_c,
                        sum(c.segundo_c) OVER (PARTITION BY c.product_id, c.year) AS mont_segundo_c,
                        sum(c.tercer_c) OVER (PARTITION BY c.product_id, c.year) AS mont_tercer_c,
                        sum(c.cuarto_c) OVER (PARTITION BY c.product_id, c.year) AS mont_cuarto_c
                       FROM ( WITH data_invoice AS (
                       SELECT account_invoice_line.product_id,
                            date_part('year'::text, account_invoice.date_invoice) AS year,
                            account_invoice.date_invoice,
                            (CASE
                            WHEN (account_invoice.type::text = 'in_invoice'::character varying)
                            THEN -account_invoice_line.price_base_currency
                            ELSE account_invoice_line.price_base_currency
                            END) AS price_base_currency,
                            0.00 AS amount_current,
                            (CASE
                            WHEN (account_invoice.type::text = 'in_invoice'::character varying)
                            THEN -account_invoice_line.price_base_currency
                            ELSE account_invoice_line.price_base_currency
                            END) AS primer_c,
                            0.00 AS segundo_c,
                            0.00 AS tercer_c,
                            0.00 AS cuarto_c
                        FROM account_invoice_line, account_invoice
                        WHERE account_invoice.id = account_invoice_line.invoice_id AND
                            (account_invoice.state::text <> ALL (ARRAY['cancel'::character varying, 'draft'::character varying]::text[]))
                            AND (account_invoice.type::text = ANY (ARRAY['in_invoice'::character varying, 'in_refund'::character varying]::text[]))
                            AND date_part('month'::text, account_invoice.date_invoice) >= 1::double precision
                            AND date_part('month'::text, account_invoice.date_invoice) <= 3::double precision
                        UNION
                        SELECT account_invoice_line.product_id,
                            date_part('year'::text, account_invoice.date_invoice) AS year,
                            account_invoice.date_invoice,
                            (CASE
                            WHEN (account_invoice.type::text = 'in_invoice'::character varying)
                            THEN -account_invoice_line.price_base_currency
                            ELSE account_invoice_line.price_base_currency
                            END) AS price_base_currency,
                            0.00 AS amount_current,
                            0.00 AS primer_c,
                            (CASE
                            WHEN (account_invoice.type::text = 'in_invoice'::character varying)
                            THEN -account_invoice_line.price_base_currency
                            ELSE account_invoice_line.price_base_currency
                            END) AS segundo_c,
                            0.00 AS tercer_c,
                            0.00 AS cuarto_c
                        FROM account_invoice_line, account_invoice
                        WHERE account_invoice.id = account_invoice_line.invoice_id AND
                            (account_invoice.state::text <> ALL (ARRAY['cancel'::character varying, 'draft'::character varying]::text[]))
                            AND (account_invoice.type::text = ANY (ARRAY['in_invoice'::character varying, 'in_refund'::character varying]::text[]))
                            AND date_part('month'::text, account_invoice.date_invoice) >= 4::double precision
                            AND date_part('month'::text, account_invoice.date_invoice) <= 6::double precision
                        UNION
                        SELECT account_invoice_line.product_id,
                            date_part('year'::text, account_invoice.date_invoice) AS year,
                            account_invoice.date_invoice,
                            (CASE
                            WHEN (account_invoice.type::text = 'in_invoice'::character varying)
                            THEN -account_invoice_line.price_base_currency
                            ELSE account_invoice_line.price_base_currency
                            END) AS price_base_currency,
                            0.00 AS amount_current,
                            0.00 AS primer_c,
                            0.00 AS segundo_c,
                            (CASE
                            WHEN (account_invoice.type::text = 'in_invoice'::character varying)
                            THEN -account_invoice_line.price_base_currency
                            ELSE account_invoice_line.price_base_currency
                            END) AS tercer_c,
                            0.00 AS cuarto_c
                        FROM account_invoice_line, account_invoice
                        WHERE account_invoice.id = account_invoice_line.invoice_id AND
                            (account_invoice.state::text <> ALL (ARRAY['cancel'::character varying, 'draft'::character varying]::text[]))
                            AND (account_invoice.type::text = ANY (ARRAY['in_invoice'::character varying, 'in_refund'::character varying]::text[]))
                            AND date_part('month'::text, account_invoice.date_invoice) >= 7::double precision
                            AND date_part('month'::text, account_invoice.date_invoice) <= 9::double precision
                        UNION
                        SELECT account_invoice_line.product_id,
                            date_part('year'::text, account_invoice.date_invoice) AS year,
                            account_invoice.date_invoice,
                            (CASE
                            WHEN (account_invoice.type::text = 'in_invoice'::character varying)
                            THEN -account_invoice_line.price_base_currency
                            ELSE account_invoice_line.price_base_currency
                            END) AS price_base_currency,
                            0.00 AS amount_current,
                            0.00 AS primer_c,
                            0.00 AS segundo_c,
                            0.00 AS tercer_c,
                            (CASE
                            WHEN (account_invoice.type::text = 'in_invoice'::character varying)
                            THEN -account_invoice_line.price_base_currency
                            ELSE account_invoice_line.price_base_currency
                            END) AS cuarto_c
                        FROM account_invoice_line, account_invoice
                        WHERE account_invoice.id = account_invoice_line.invoice_id AND
                            (account_invoice.state::text <> ALL (ARRAY['cancel'::character varying, 'draft'::character varying]::text[]))
                            AND (account_invoice.type::text = ANY (ARRAY['in_invoice'::character varying, 'in_refund'::character varying]::text[]))
                            AND date_part('month'::text, account_invoice.date_invoice) >= 10::double precision
                            AND date_part('month'::text, account_invoice.date_invoice) <= 12::double precision
                        UNION
                        SELECT account_invoice_line.product_id,
                            date_part('year'::text, account_invoice.date_invoice) AS year,
                            account_invoice.date_invoice,
                            (CASE
                            WHEN (account_invoice.type::text = 'in_invoice'::character varying)
                            THEN -account_invoice_line.price_base_currency
                            ELSE account_invoice_line.price_base_currency
                            END) AS price_base_currency,
                            (CASE
                            WHEN (account_invoice.type::text = 'in_invoice'::character varying)
                            THEN -account_invoice_line.price_base_currency
                            ELSE account_invoice_line.price_base_currency
                            END) AS amount_current,
                            0.00 AS primer_c,
                            0.00 AS segundo_c,
                            0.00 AS tercer_c,
                            0.00 AS cuarto_c
                        FROM account_invoice_line, account_invoice
                        WHERE account_invoice.id = account_invoice_line.invoice_id AND
                            (account_invoice.state::text <> ALL (ARRAY['cancel'::character varying, 'draft'::character varying]::text[]))
                            AND (account_invoice.type::text = ANY (ARRAY['in_invoice'::character varying, 'in_refund'::character varying]::text[]))
                            AND date_part('month'::text, account_invoice.date_invoice) = date_part('month'::text, 'now'::text::date))
                             SELECT di.product_id,
                                di.year,
                                di.date_invoice,
                                di.price_base_currency,
                                di.amount_current,
                                di.primer_c,
                                di.segundo_c,
                                di.tercer_c,
                                di.cuarto_c
                               FROM data_invoice di
                              ORDER BY di.date_invoice, di.product_id) c
                                )
                         SELECT row_number() over() as id,
                                prodt.categ_id,
                                source.year,
                                source.mont_total,
                                source.mont_current,
                                source.mont_primer_c,
                                source.mont_segundo_c,
                                source.mont_tercer_c,
                                source.mont_cuarto_c
                           FROM source, product_product prod, product_template prodt
                          WHERE prod.id = source.product_id and prod.product_tmpl_id = prodt.id
                          GROUP BY prodt.categ_id, source.year, source.mont_total, source.mont_current, source.mont_primer_c, source.mont_segundo_c, source.mont_tercer_c, source.mont_cuarto_c
                          ORDER BY source.year desc, source.mont_total desc
            )
        """)
travel_product_purchase_invoice_report()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
