# -*- coding: utf-8 -*-

from openerp.osv import fields, osv
import time
import xlwt


class ExcelWizard(osv.osv_memory):
    _name = 'excel.wizard'

    _columns = {
        'report': fields.selection(
            (('gl', 'General Ledger'), ('tb', 'Trial Balance'), ('bs', 'Balance Sheet'), ('pl', 'Profit Loss')),
            'Report'),
        'fiscalyear_id': fields.many2one('account.fiscalyear', 'Fiscal Year', required=True),
        'period_from': fields.many2one('account.period', 'Start Period',
                                       domain="[('fiscalyear_id', '=', fiscalyear_id)]", required=True),
        'period_to': fields.many2one('account.period', 'End Period', domain="[('fiscalyear_id', '=', fiscalyear_id)]",
                                     required=True),
        'name': fields.char('File Name', 16),
    }

    _defaults = {
        'report': 'bs',
        'fiscalyear_id': 1,
        'period_from': 2,
        'period_to': 12,
    }

    def export_excel(self, cr, uid, ids, context=None):
        data = []
        val = self.browse(cr, uid, ids)[0]
        obj_account = self.pool.get('account.account')
        obj_move = self.pool.get('account.move')

        # Hacemos una regla para asegurarnos de que el usuario ingrese el período correcto en orden
        if val.period_from.id > val.period_to.id:
            raise osv.except_osv(('Error'), ('Start period should be smaller then End period'))

        # Esta regla sirve para garantizar que ya exista una cuenta con el nombre BALANCE SHEET en el CoA
        bsid = obj_account.search(cr, uid, [('name', '=', 'Balance Sheet')])
        if not bsid:
            raise osv.except_osv(('Error'), ('You must create BALANCE SHEET account'))

        # Consulta todas las jerarquías de la cuenta BALANCE SHEET y recupera los datos
        ids_acc = obj_account._get_children_and_consol(cr, uid, [bsid[0]])
        acc_data = obj_account.browse(cr, uid, ids_acc)

        # Tome todas las transacciones que están en todos los diarios relacionados con la cuenta anterior
        mid = obj_move.search(cr, uid, [('state', '=', 'posted'), (
            'period_id', 'in', tuple([x.id for x in val.fiscalyear_id.period_ids if x.id <= val.period_from.id]))])

        # Realice un bucle en todas las cuentas y tome la cantidad de mutación (débito y crédito) con el método self.get_amount ()
        for acc in acc_data:
            if acc.type == 'view':
                ending = self.get_amount(cr, uid, ids, acc, mid)
                data.append([acc.code, acc.name, ending[0] - ending[1]])

        if context is None:
            context = {}

        # Preparar parámetros para el informe de Excel
        nilai = self.read(cr, uid, ids)[0]
        datas = {'ids': [nilai['id']]}
        datas['model'] = 'accounting.legal'
        datas['form'] = nilai
        datas['csv'] = data

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'balance.sheet.excel',
            'nodestroy': True,
            'datas': datas,
        }

    def get_amount(self, cr, uid, ids, account, move):
        ids_acc = self.pool.get('account.account')._get_children_and_consol(cr, uid, [account.id])
        acc_data = self.pool.get('account.account').browse(cr, uid, ids_acc)
        debit = 0;
        credit = 0
        for a in acc_data:
            if a.type != 'view':
                lid = self.pool.get('account.move.line').search(cr, uid,
                                                                [('move_id', 'in', move), ('account_id', '=', a.id)])
                lad = self.pool.get('account.move.line').browse(cr, uid, lid)
                debit += sum([x.debit for x in lad])
                credit += sum([x.credit for x in lad])
        return debit, credit
