# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv
from openerp.osv.orm import TransientModel
from string import upper


class fix_database(TransientModel):
    _name = 'fix.database'
    _columns = {
        'action':
            fields.selection([
                   ('unique_partner', 'Unique Partner'),
                   ('unify_invoice_due_date', 'Unify invoice and due date'),
                   ('unify_invoice_origin', 'Unify invoice origin'),
                   ('clean_prices', 'Clean prices'),
                   ('fill_descriptions', 'Fill descriptions'),
                   ('set_user_to_moves', 'Set user to moves'),
                   ('set_internal_customer_to_moves', 'Set internal customer to moves'),
                   ('set_prices_for_margin', 'Set prices for margin'),
                   ('set_prices_for_sale_base_currency', 'Set prices for sale base currency'),
                   ('set_prices_for_invoice_base_currency', 'Set prices for invoice base currency'),
                   ('set_tax_to_products', 'Set tax to products'),
                   ('unique_product_hotel', 'Unique Product Hotel'),
                   ], 'Action'),
        # 'good_name': fields.char('Good Name', size=255),
        # 'bad_name': fields.char('Bad Name', size=255),
        'product_hotel_id': fields.many2one('product.hotel', 'Good Hotel', ondelete="cascade"),
        'product_hotel_ids': fields.many2many('product.hotel', 'fix_database_product_hotel_rel', 'fix_database_id',
                                              'product_hotel_id', 'Bad Hotels', ondelete="cascade"),
        'partner_id': fields.many2one('res.partner', 'Good Partner', ondelete="cascade"),
        'partner_ids': fields.many2many('res.partner', 'fix_database_res_partner_rel', 'fix_database_id',
                                              'partner_id', 'Bad Partners', ondelete="cascade")
    }

    def execute_action(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context)
        context['object'] = obj
        res = getattr(self, obj.action)(cr, uid, context)
        if obj.action == 'unique_product_hotel':
            context = {}
            return {
                'name': 'Fix Database',
                'type': 'ir.actions.act_window',
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'fix.database',
                'action': 'unique_product_hotel',
                'target': 'new',
                'context': context,
            }
        return res

    def set_tax_to_products(self, cr, uid, ids, context=None):
        product = self.pool.get('product.product')
        tax = self.pool.get('account.tax')
        tax_id = tax.search(cr, uid, [], context=context)
        if tax_id:
            product_ids = product.search(cr, uid, [], context=context)
            product.write(cr, uid, product_ids, {'taxes_id': [(6, 0, tax_id)]}, context)
        return True

    def set_prices_for_margin(self, cr, uid, context=None):
        inv = self.pool.get('account.invoice')
        to_search_sale = [
            ('type', '=', 'out_invoice'),
            ('state', 'not in', ['draft', 'cancel'])
        ]
        inv_ids = inv.search(cr, uid, to_search_sale, context=context)
        for obj in inv.browse(cr, uid, inv_ids, context):

            usd_sp = inv.get_in_base_currency(cr, uid, obj.amount_total, obj, context)
            #usd_sp_refund = self.get_refund_amount(cr, uid, 'out_refund', obj, context)
            #usd_sp = usd_sp - usd_sp_refund

            to_search_cost = [
                ('origin', '=', obj.origin),
                ('id', '!=', obj.id),
                ('type', '=', 'in_invoice'),
                ('state', 'not in', ['draft', 'cancel'])
            ]

            cost_inv_ids = inv.search(cr, uid, to_search_cost, context=context)
            usd_cp = 0.0
            for x in inv.browse(cr, uid, cost_inv_ids):
                usd_cp += inv.get_in_base_currency(cr, uid, x.amount_total, x, context)
                #usd_cp -= self.get_refund_amount(cr, uid, 'in_refund', x, context)

            # Esto es un parche a falta de una buena solucion
            if usd_sp == 10.00 and obj.origin == 'CM01152':
                usd_cp = 0.0

            usd_margin = 0.0
            if usd_sp > 0:
                usd_margin = 100 - (usd_cp / usd_sp * 100)
            cr.execute('update account_invoice set sale_price=%s, cost_price=%s, margin=%s where id=%s', (usd_sp, usd_cp, usd_margin, obj.id))
        return True

    def get_refund_amount(self, cr, uid, inv_type, obj, context):
        inv = self.pool.get('account.invoice')
        usd_refund = 0.0
        to_search_refund = [
            ('type', '=', inv_type),
            ('state', 'not in', ['draft', 'cancel']),
            ('origin', '=', obj.origin)
        ]
        refund_inv_ids = inv.search(cr, uid, to_search_refund, context=context)
        product_ids = [l.product_id.id for l in obj.invoice_line]
        for f in inv.browse(cr, uid, refund_inv_ids):
            refund_product_ids = [l.product_id.id for l in f.invoice_line]
            if set(refund_product_ids).issubset(set(product_ids)):
                usd_refund += inv.get_in_base_currency(cr, uid, f.amount_total, f, context)
        return usd_refund

    def set_prices_for_invoice_base_currency(self, cr, uid, context=None):
        currency = self.pool.get('res.currency')

        invoice_line = self.pool.get('account.invoice.line')
        invoice_line_ids = invoice_line.search(cr, uid, [], context=context)
        for obj in invoice_line.browse(cr, uid, invoice_line_ids, context):
            value = 0.00
            current_price = obj.price_unit * obj.quantity
            base_currency_id = obj.invoice_id.company_id.currency_id.id
            invoice_currency_id = obj.invoice_id.currency_id.id
            if base_currency_id != invoice_currency_id:
                context['date'] = obj.invoice_id.date_invoice
                value = round(currency.compute(cr, uid, invoice_currency_id, base_currency_id, current_price,
                                               round=True, context=context), 2)
            else:
                value = round(current_price, 2)
            cr.execute('update account_invoice_line set price_base_currency=%s where id=%s', (value, obj.id))

        invoice = self.pool.get('account.invoice')
        invoice_ids = invoice.search(cr, uid, [], context=context)
        for obj in invoice.browse(cr, uid, invoice_ids, context):
            value = round(sum([x.price_base_currency for x in obj.invoice_line]), 2)
            cr.execute('update account_invoice set total_base_currency=%s where id=%s', (value, obj.id))

        return True

    def set_prices_for_sale_base_currency(self, cr, uid, context=None):
        currency = self.pool.get('res.currency')

        order_line = self.pool.get('sale.order.line')
        order_line_ids = order_line.search(cr, uid, [], context=context)
        for obj in order_line.browse(cr, uid, order_line_ids, context):
            value = 0.00
            actual_price = obj.price_unit * obj.product_uom_qty
            base_currency_id = obj.order_id.company_id.currency_id.id
            order_currency_id = obj.order_id.pricelist_id.currency_id.id
            if base_currency_id != order_currency_id:
                context['date'] = obj.order_id.date_order
                value = round(currency.compute(cr, uid, order_currency_id, base_currency_id, actual_price, round=True,
                                         context=context), 2)
            else:
                value = round(actual_price, 2)
            cr.execute('update sale_order_line set price_base_currency=%s where id=%s', (value, obj.id))

        order = self.pool.get('sale.order')
        order_ids = order.search(cr, uid, [], context=context)
        for obj in order.browse(cr, uid, order_ids, context):
            value = round(sum([x.price_base_currency for x in obj.order_line]), 2)
            cr.execute('update sale_order set total_base_currency=%s where id=%s', (value, obj.id))

        return True

    def set_user_to_moves(self, cr, uid, context=None):
        acl = self.pool.get('account.move.line')
        acl_ids = acl.search(cr, uid, [], context=context)
        for obj in acl.browse(cr, uid, acl_ids, context):
            if obj.invoice:
                user_id = obj.invoice.user_id.id
                cr.execute('update account_move_line set user_id=%s where id=%s', (user_id, obj.id))
        return True

    def set_internal_customer_to_moves(self, cr, uid, context=None):
        acl = self.pool.get('account.move.line')
        acl_ids = acl.search(cr, uid, [], context=context)
        for obj in acl.browse(cr, uid, acl_ids, context):
            value = 'CUSTOMER'
            if obj.invoice:
                inv = obj.invoice
                tags = []
                if obj.invoice.type in ['out_invoice', 'out_refund']:
                    tags = [x.name for x in inv.partner_id.category_id]
                elif inv.other_partner_id:
                    tags = [x.name for x in inv.other_partner_id.category_id]
                if 'CEIBA' in tags:
                    value = 'CEIBA'
                cr.execute('update account_move_line set internal_customer=%s where id=%s', (value, obj.id))
        return True

    def fill_descriptions(self, cr, uid, context=None):
        sol = self.pool.get('sale.order.line')
        sol_ids = sol.search(cr, uid, [('description', '=', False)],
                             context=context)
        for obj in sol.browse(cr, uid, sol_ids, context):
            if not obj.description:
                to_write = {'description': obj.name}
                sol.write(cr, uid, [obj.id], to_write, context)
        return True

    def clean_prices(self, cr, uid, context=None):
        ppi = self.pool.get('pricelist.partnerinfo')
        categs = ['Program', 'Hotel', 'Transfer', 'Car']
        ppi_ids = ppi.search(cr, uid, [('suppinfo_id.product_id.categ_id.name', 'in', categs)], context=context)
        ppi.unlink(cr, uid, ppi_ids, context)
        return True

    def unify_invoice_due_date(self, cr, uid, context=None):
        inv = self.pool.get('account.invoice')
        inv_ids = inv.search(cr, uid, [('state', 'not in', ['draft', 'cancel'])], context=context)
        for obj in inv.browse(cr, uid, inv_ids, context):
            date_due = obj.date_invoice
            inv.write(cr, uid, [obj.id], {'date_due': date_due}, context)
        return True

    def unify_invoice_origin(self, cr, uid, context=None):
        inv = self.pool.get('account.invoice')
        invline = self.pool.get('account.invoice.line')
        inv_ids = inv.search(cr, uid, [('state', 'not in', ['draft', 'cancel']),('type','in',['in_invoice','in_refund'])], context=context)
        for obj in inv.browse(cr, uid, inv_ids, context):
            for line in obj.invoice_line:
                if not obj.origin and line.origin:
                    fact = inv.search(cr, uid, [('number', '=', line.origin)], context=context)[0]
                    if fact:
                        fact_obj = inv.browse(cr, uid, fact, context)
                        cr.execute('update account_invoice set origin=%s where id=%s', (fact_obj.origin, fact_obj.id))
                    else:
                        cr.execute('update account_invoice set origin=%s where id=%s', (line.origin, obj.id))
                elif obj.origin and not line.origin:
                    cr.execute('update account_invoice_line set origin=%s where id=%s', (obj.origin, line.id))
                elif obj.origin != line.origin:
                    cr.execute('update account_invoice_line set origin=%s where id=%s', (obj.origin, line.id))
                elif not obj.origin and not line.origin or obj.origin == line.origin:
                    pass
                else:
                    raise osv.except_osv('Error', 'Unknown')
        return True

    def unique_partner(self, cr, uid, context=None):
        partner = self.pool.get('res.partner')
        obj = context['object']
        bad_partner_id = []
        try:
            bad_partner_id = partner.search(cr, uid, [('name', 'ilike', upper(obj.partner_id.name))], context=context)
            good_partner_id = obj.partner_id.id
        except:
            raise osv.except_osv('Error', 'The name is not correct.')
        if obj.partner_ids:
            for part in obj.partner_ids:
                if part.id not in bad_partner_id:
                    bad_partner_id.append(part.id)
        if good_partner_id in bad_partner_id:
            bad_partner_id.remove(good_partner_id)
        partners = [(bad_partner_id, good_partner_id)]
        for p in partners:
            for x in p[0]:
                cr.execute('update sale_order set partner_id=%s, partner_invoice_id=%s, partner_shipping_id=%s where partner_id=%s', (p[1], p[1], p[1], x))
                cr.execute('update sale_order_line set customer_id=%s, order_partner_id=%s where customer_id=%s', (p[1], p[1], x))
                cr.execute('update sale_order_line set supplier_id=%s where supplier_id=%s', (p[1], x))
                cr.execute('update account_voucher set partner_id=%s where partner_id=%s', (p[1], x))
                cr.execute('update account_invoice set partner_id=%s, commercial_partner_id=%s where partner_id=%s', (p[1], p[1], x))
                cr.execute('update account_invoice_line set partner_id=%s where partner_id=%s', (p[1], x))
                cr.execute('update account_move set partner_id=%s where partner_id=%s', (p[1], x))
                cr.execute('update account_move_line set partner_id=%s where partner_id=%s', (p[1], x))
                cr.execute('update account_model_line set partner_id=%s where partner_id=%s', (p[1], x))
                cr.execute('update product_program_line set supplier_id=%s where supplier_id=%s', (p[1], x))
                self.pool.get('res.partner').unlink(cr, uid, [x], context)
        return True

    def unique_product_hotel(self, cr, uid, context=None):
        product = self.pool.get('product.product')
        product_hotel = self.pool.get('product.hotel')
        obj = context['object']
        try:
            bad_product_id = product.search(cr, uid, [('name', 'ilike', upper(obj.product_hotel_id.name))], context=context)
            good_product_id = obj.product_hotel_id.product_id.id
            good_product_name = obj.product_hotel_id.product_id.name
            if good_product_id in bad_product_id:
                bad_product_id.remove(good_product_id)
        except:
            raise osv.except_osv('Error', 'The name is not correct.')

        black_list = []
        if obj.product_hotel_ids:
            for bh in obj.product_hotel_ids:
                if good_product_id != bh.product_id.id:
                    black_list.append((bh.product_id.id, bh.product_id.product_tmpl_id.uom_id.id,
                                       bh.product_id.product_tmpl_id.uos_id.id))

        if bad_product_id:
            n = [v[0] for v in black_list]
            for bad in product.browse(cr, uid, bad_product_id, context):
                if bad.id not in n:
                    black_list.append((bad.id, bad.product_tmpl_id.uom_id.id, bad.product_tmpl_id.uos_id.id))

        if not bad_product_id and not obj.product_hotel_ids:
            raise osv.except_osv('Error', 'The name is not duplicate.')

        products = [(black_list, good_product_id, good_product_name)]
        for p in products:
            for x in p[0]:

                cr.execute('update sale_order_line set product_id=%s, product_uom=%s, product_uos=%s where product_id=%s', (p[1], x[1] or None, x[2] or None, x[0]))
                cr.execute('SELECT account_invoice.comment, account_invoice_line.name, account_invoice_line.invoice_id FROM '
                            'public.account_invoice, public.account_invoice_line WHERE '
                            'account_invoice.id = account_invoice_line.invoice_id AND account_invoice_line.product_id =%s', (x[0],))
                for (comment, name, invoice_id) in cr.fetchall():
                    description = 'Note:' + (comment or ' Change name product') + '. Change the line name product(' + name + ') by ' + p[2] + '.'
                    cr.execute('update account_invoice set comment=%s, name=%s where id=%s', (description, p[2], invoice_id))
                cr.execute('update account_invoice_line set product_id=%s, uos_id=%s where product_id=%s', (p[1], x[1] or None, x[0]))
                cr.execute('SELECT account_move.narration, account_move_line.name, account_move_line.move_id FROM '
                            'public.account_move, public.account_move_line WHERE '
                            'account_move.id = account_move_line.move_id AND account_move_line.product_id =%s', (x[0],))
                for (narration, name, move_id) in cr.fetchall():
                    description = 'Note:' + (narration or ' Change name product') + '. Change the line name product(' + name + ') by ' + p[2] + '.'
                    cr.execute('update account_move set narration=%s where id=%s', (description, move_id))
                cr.execute('update account_move_line set product_id=%s, product_uom_id=%s, name=%s where product_id=%s', (p[1], x[1] or None, p[2], x[0]))
                cr.execute('update account_analytic_line set product_id=%s, product_uom_id=%s where product_id=%s', (p[1], x[1] or None, x[0]))
                cr.execute('update product_pricelist_item set product_id=%s where product_id=%s', (p[1], x[0]))
                cr.execute('update product_supplierinfo set product_id=%s where product_id=%s', (p[1], x[0]))

            self.pool.get('product.hotel').unlink(cr, uid, [x[0] for x in p[0]], context)
            self.pool.get('product.product').unlink(cr, uid, [x[0] for x in p[0]], context)
            self.pool.get('product.template').unlink(cr, uid, [x[0] for x in p[0]], context)
        return True
