# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import netsvc
from openerp.osv import fields
from openerp.osv.orm import Model, TransientModel
import openerp.addons.decimal_precision as dp


class res_partner(Model):
    _inherit = 'res.partner'
    _columns = {
        'account_customer_deposit_id': fields.property(
            'account.account',
            type='many2one',
            relation='account.account',
            string="Customer Deposit Account",
            view_load=True,
            domain="[('type', '=', 'receivable')]")
    }


class sale_order(Model):
    _inherit = 'sale.order'

    def _get_advanced(self, cr, uid, ids, fields, args, context=None):
        res = {}
        absl = self.pool.get('account.bank.statement.line')
        for obj in self.browse(cr, uid, ids, context):
            res[obj.id] = 0.0
            to_search = [('name', '=', 'Advanced payment for ' + obj.name)]
            absl_ids = absl.search(cr, uid, to_search, context=context)
            if absl_ids:
                res[obj.id] = sum([x.amount for x in absl.browse(cr, uid, absl_ids)])
        return res

    _columns = {
        'advanced': fields.function(_get_advanced, method=True, type='float',
                                    digits_compute=dp.get_precision('Account'),
                                    string='Advanced')
    }

    def order_advanced_pay(self, cr, uid, ids, context=None):
        order = self.browse(cr, uid, ids[0], context=context)
        ap_obj = self.pool.get('advanced.payment')
        vals = {
            'reference': 'Advanced payment for ' + order.name,
            'partner_id': order.partner_id.id,
            'amount': order.amount_total - order.advanced,
            'journal_id': 14
        }
        ap_id = ap_obj.create(cr, uid, vals, context)
        return {
            'name': "Advanced Payment",
            'view_mode': 'form',
            'view_type': 'form',
            'res_model': 'advanced.payment',
            'res_id': ap_id,
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'new'
        }


class advanced_payment(TransientModel):
    _name = 'advanced.payment'
    _columns = {
        'reference': fields.char('Reference', size=128),
        'partner_id': fields.many2one('res.partner', 'Partner', required=True),
        'journal_id': fields.many2one('account.journal', 'Payment Method',
                                      required=True),
        'amount': fields.float('Amount',
                                digits_compute=dp.get_precision('Account'),
                                required=True),
    }

    def create_bank_statement(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context)
        bs = self.pool.get('account.bank.statement')

        journal_id = obj.journal_id.id
        balance_start = bs._compute_balance_end_real(cr, uid, journal_id, context)
        balance_end_real = balance_start + obj.amount

        vals = {
            'journal_id': journal_id,
            'balance_start': balance_start,
            'balance_end_real': balance_end_real,
            'line_ids': [(0, 0, {
                'name': obj.reference,
                'amount': obj.amount,
                'partner_id': obj.partner_id.id,
                'account_id': obj.partner_id.account_customer_deposit_id.id
            })]
        }
        bs.create(cr, uid, vals, context)

        order = self.pool.get('sale.order').browse(cr, uid, context['active_id'])
        if order.amount_total == order.advanced:
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'sale.order', context['active_id'], 'order_confirm', cr)
        return True
