# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields
from openerp.osv.orm import Model

import openerp.addons.decimal_precision as dp


class account_invoice(Model):
    _name = 'account.invoice'
    _inherit = 'account.invoice'

    def _get_total_base_currency(self, cr, uid, ids, fields, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            res[obj.id] = sum([x.price_base_currency for x in obj.invoice_line])
        return res

    def _get_invoice_line(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('account.invoice.line').browse(cr, uid, ids, context=context):
            result[line.invoice_id.id] = True
        return result.keys()

    _columns = {
        'total_base_currency': fields.function(_get_total_base_currency,
                string='Total (USD)',
                digits_compute=dp.get_precision('Account'),
                store={
                    'account.invoice': (lambda self, cr, uid, ids, c={}: ids, ['invoice_line'], 10),
                    'account.invoice.line': (_get_invoice_line, ['price_unit','invoice_line_tax_id','quantity','discount','invoice_id'], 20),
                })
    }


class account_invoice_line(Model):
    _name = 'account.invoice.line'
    _inherit = 'account.invoice.line'

    def _get_price_base_currency(self, cr, uid, ids, fields, args, context=None):
        cr_obj = self.pool.get('res.currency')
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            current_price = obj.price_unit * obj.quantity
            res[obj.id] = current_price
            if obj.invoice_id:
                base_currency_id = obj.invoice_id.company_id.currency_id.id
                invoice_currency_id = obj.invoice_id.currency_id.id
                if base_currency_id != invoice_currency_id:
                    context['date'] = obj.invoice_id.date_invoice
                    res[obj.id] = cr_obj.compute(cr, uid, invoice_currency_id, base_currency_id, current_price, round=False, context=context)
        return res

    _columns = {
        'price_base_currency': fields.function(_get_price_base_currency,
                string='Subtotal',
                digits_compute=dp.get_precision('Account'),
                store={
                    'account.invoice.line': (lambda self, cr, uid, ids, c={}: ids, ['price_unit', 'quantity'], 10),
                })
    }
