# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2013, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import tools
from openerp.osv import fields
from openerp.osv.orm import Model
from openerp.tools import float_round


class account_move_line(Model):
    _name = 'account.move.line'
    _inherit = 'account.move.line'

#    def create(self, cr, uid, vals, context=None, check=True):
#        res = super(account_move_line, self).create(cr, uid, vals, context, check)
#        obj = self.browse(cr, uid, res, context)
#        if obj.journal_id.code in ['SAJ', 'SCNJ']:
#            self.amount_with_better_rate(cr, uid, obj, context)
#        return res
#
#    def write(self, cr, uid, ids, vals, context=None, check=True, update_check=True):
#        res = super(account_move_line, self).write(cr, uid, ids, vals, context, check, update_check)
#        for obj in self.browse(cr, uid, ids, context):
#            if obj.journal_id.code in ['SAJ', 'SCNJ']:
#                self.amount_with_better_rate(cr, uid, obj, context)
#        return res

    # TODO: cambiar el 1 por el rate de la moneda base
    def amount_with_better_rate(self, cr, uid, aml, context=None):
        if aml.currency_id and aml.amount_currency:
            to_currency = aml.company_id.currency_id
            better_rate = [r for r in aml.currency_id.rate_ids if r.name <= aml.date][0]
            base_amount = aml.amount_currency * (1 / better_rate.bad_rate)
            base_amount = float_round(base_amount, precision_rounding=to_currency.rounding)
            debit, credit = 0.0, 0.0
            if base_amount > 0:
                debit = base_amount
            else:
                credit = abs(base_amount)
            cr.execute('update account_move_line set debit=%s, credit=%s where id=%s', (debit, credit, aml.id))
        return True

    def _get_user(self, cr, uid, ids, fields, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            res[obj.id] = False
            if obj.invoice:
                res[obj.id] = obj.invoice.user_id.id
        return res

    def _is_internal(self, cr, uid, ids, field, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            res[obj.id] = False
            if obj.invoice:
                tags = []
                inv = obj.invoice
                if inv.type in ['out_invoice', 'out_refund']:
                    tags = [x.name for x in inv.partner_id.category_id]
                elif inv.other_partner_id:
                    tags = [x.name for x in inv.other_partner_id.category_id]
                res[obj.id] = 'CUSTOMER'
                if 'CEIBA' in tags:
                    res[obj.id] = 'CEIBA'
        return res

    def _get_source(self, cr, uid, ids, field, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            res[obj.id] = False
            if obj.invoice:
                res[obj.id] = obj.invoice.origin
        return res

    _columns = {
        'create_uid': fields.many2one('res.users', 'Owner', readonly=True),
        # 'user_id': fields.function(_get_user, method=True, type='many2one',
        #                           string='User', relation='res.users',
        #                           store=True),
        'user_id': fields.many2one('res.users', 'User', help="The user responsible for this journal entries"),
        'internal_customer': fields.function(_is_internal, method=True,
                                             type='char', size=128,
                                             string='Internal', store=True),
        'sale_order': fields.function(_get_source, method=True, type='char',
                                      size=255, string='Source')
    }

    _defaults = {
        'user_id': lambda self, cr, uid, context: uid,
    }


class account_voucher(Model):
    _name = 'account.voucher'
    _inherit = 'account.voucher'

    def action_move_line_create(self, cr, uid, ids, context=None):
        '''
        Confirm the vouchers given in ids and create the journal entries for each of them
        '''
        if context is None:
            context = {}
        move_pool = self.pool.get('account.move')
        move_line_pool = self.pool.get('account.move.line')
        for voucher in self.browse(cr, uid, ids, context=context):
            local_context = dict(context, force_company=voucher.journal_id.company_id.id)
            if voucher.move_id:
                continue
            company_currency = self._get_company_currency(cr, uid, voucher.id, context)
            current_currency = self._get_current_currency(cr, uid, voucher.id, context)
            # we select the context to use accordingly if it's a multicurrency case or not
            context = self._sel_context(cr, uid, voucher.id, context)
            # But for the operations made by _convert_amount, we always need to give the date in the context
            ctx = context.copy()
            ctx.update({'date': voucher.date})
            # Create the account move record.
            move_id = move_pool.create(cr, uid, self.account_move_get(cr, uid, voucher.id, context=context), context=context)
            # Get the name of the account_move just created
            name = move_pool.browse(cr, uid, move_id, context=context).name
            # Create the first line of the voucher
            move_line_id = move_line_pool.create(cr, uid, self.first_move_line_get(cr,uid,voucher.id, move_id, company_currency, current_currency, local_context), local_context)
            move_line_brw = move_line_pool.browse(cr, uid, move_line_id, context=context)
            line_total = move_line_brw.debit - move_line_brw.credit
            rec_list_ids = []
            if voucher.type == 'sale':
                line_total = line_total - self._convert_amount(cr, uid, voucher.tax_amount, voucher.id, context=ctx)
            elif voucher.type == 'purchase':
                line_total = line_total + self._convert_amount(cr, uid, voucher.tax_amount, voucher.id, context=ctx)
            # Create one move line per voucher line where amount is not 0.0
            line_total, rec_list_ids = self.voucher_move_line_create(cr, uid, voucher.id, line_total, move_id, company_currency, current_currency, context)

            # Create the writeoff line if needed
            ml_writeoff = self.writeoff_move_line_get(cr, uid, voucher.id, line_total, move_id, name, company_currency, current_currency, local_context)
            if ml_writeoff:
                move_line_pool.create(cr, uid, ml_writeoff, local_context)
            # We post the voucher.
            self.write(cr, uid, [voucher.id], {
                'move_id': move_id,
                'state': 'posted',
                'number': name,
            })
            if voucher.journal_id.entry_posted:
                move_pool.post(cr, uid, [move_id], context={})
            # We automatically reconcile the account move lines.
            reconcile = False
            for rec_ids in rec_list_ids:
                if len(rec_ids) >= 2:
                    reconcile = move_line_pool.reconcile_partial(cr, uid, rec_ids, writeoff_acc_id=voucher.writeoff_acc_id.id, writeoff_period_id=voucher.period_id.id, writeoff_journal_id=voucher.journal_id.id)

            # Post the move
            move_pool.button_validate(cr, uid, [move_id], context=context)
        return True


class account_voucher_line(Model):
    _name = 'account.voucher.line'
    _inherit = 'account.voucher.line'
    _columns = {
        'sale_order': fields.related('move_line_id', 'sale_order', type='char',
                                     string='Source', readonly=True)
    }


class account_entries_report(Model):
    _name = "account.entries.report"
    _inherit = "account.entries.report"
    _columns = {
        'user_id': fields.many2one('res.users', 'Salesperson'),
        'internal_customer': fields.char('Internal Customer', size=128)
    }

    def init(self, cr):
        tools.drop_view_if_exists(cr, 'account_entries_report')
        cr.execute("""
            create or replace view account_entries_report as (
            select
                l.id as id,
                am.date as date,
                l.date_maturity as date_maturity,
                l.date_created as date_created,
                am.ref as ref,
                am.state as move_state,
                l.state as move_line_state,
                l.reconcile_id as reconcile_id,
                to_char(am.date, 'YYYY') as year,
                to_char(am.date, 'MM') as month,
                to_char(am.date, 'YYYY-MM-DD') as day,
                l.partner_id as partner_id,
                l.product_id as product_id,
                l.product_uom_id as product_uom_id,
                am.company_id as company_id,
                am.journal_id as journal_id,
                p.fiscalyear_id as fiscalyear_id,
                am.period_id as period_id,
                l.account_id as account_id,
                l.analytic_account_id as analytic_account_id,
                a.type as type,
                a.user_type as user_type,
                1 as nbr,
                l.quantity as quantity,
                l.currency_id as currency_id,
                l.amount_currency as amount_currency,
                l.debit as debit,
                l.credit as credit,
                l.debit-l.credit as balance,
                l.user_id as user_id,
                l.internal_customer as internal_customer
            from
                account_move_line l
                left join account_account a on (l.account_id = a.id)
                left join account_move am on (am.id=l.move_id)
                left join account_period p on (am.period_id=p.id)
                where l.state != 'draft'
            )
        """)
account_entries_report()
