# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields
from openerp.osv.orm import Model
import time
from datetime import datetime


class res_currency(Model):
    _name = 'res.currency'
    _inherit = 'res.currency'
    _columns = {
        'month_rate': fields.float('Month Rate', digits=(12, 6))
    }

    def write(self, cr, uid, ids, vals, context=None):
        if not context:
            context = {}
        if not context.get('flag_rate', False):
            res = super(res_currency, self).write(cr, uid, ids, vals, context=context)
            if vals.get('month_rate', False):
                cur_rate_obj = self.pool.get('res.currency.rate')
                month_curr = datetime.today().month
                for r in self.browse(cr, uid, ids, context=context):
                    rate_ids = []
                    if r.rate_ids:
                        for y in r.rate_ids:
                            rate_ids.append(y)
                rate_type = self.pool.get('res.currency.rate.type')
                for rec in rate_ids:
                    if datetime.strptime(rec.name, '%Y-%m-%d').month == month_curr:
                        vals_rate = {'rate': vals.get('month_rate', False)}
                        context.update({'flag_rate': True})
                        cur_rate_obj.write(cr, uid, [rec.id], vals_rate, context=context)
        res = super(res_currency, self).write(cr, uid, ids, vals, context=context)
        return res


class res_currency_rate(Model):
    _name = 'res.currency.rate'
    _inherit = 'res.currency.rate'

    # _defaults = {
    #     'currency_rate_type_id': 1
    # }

    def update_rate(self, cr, uid, ids, context=None):
        for obj in self.browse(cr, uid, ids, context):
            if context.get('rates', False):
                rates = context.get('rates', False)
                good_rate = min(rates.get(obj.id), obj.currency_id.month_rate)
            else:
                good_rate = min(obj.rate, obj.currency_id.month_rate)
            rate_type = self.pool.get('res.currency.rate.type')
            currency_rate_type_id = rate_type.search(cr, uid, [('name', '=', 'Sale')])[0]
            if not obj.currency_rate_type_id:
                cr.execute('update res_currency_rate set rate=%s where id=%s', (obj.currency_id.month_rate, obj.id))
            elif obj.currency_rate_type_id.id != currency_rate_type_id:
                cr.execute('update res_currency_rate set rate=%s where id=%s', (obj.currency_id.month_rate, obj.id))
            elif obj.currency_rate_type_id.id == currency_rate_type_id:
                cr.execute('update res_currency_rate set rate=%s where id=%s', (good_rate, obj.id))

            exist = self.search(cr, uid, [('name', '=', obj.name), ('currency_rate_type_id', '=', currency_rate_type_id), ('currency_id', '=', obj.currency_id.id)])
            if not exist:
                cr.execute('insert into res_currency_rate (name, rate, currency_id, currency_rate_type_id) values (%s, %s, %s, %s)', (obj.name, good_rate, obj.currency_id.id, currency_rate_type_id))
        return True

    def create(self, cr, uid, vals, context=None):
        res_id = super(res_currency_rate, self).create(cr, uid, vals, context)
        if not context:
            context = {}
        self.update_rate(cr, uid, [res_id], context)
        return res_id

    def write(self, cr, uid, ids, vals, context=None):
        if not context:
            context = {}

        if context.get('flag_rate', False):
            rates = {}
            for obj in self.browse(cr, uid, ids, context):
                rates.update({obj.id: obj.rate})
            context.update({'rates': rates})
        res = super(res_currency_rate, self).write(cr, uid, ids, vals, context)
        self.update_rate(cr, uid, ids, context)
        return res