# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2013, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields
from openerp.osv.orm import Model
from openerp import netsvc


class account_invoice(Model):
    _name = 'account.invoice'
    _inherit = 'account.invoice'

    def _get_other_partner(self, cr, uid, ids, fields, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            res[obj.id] = False
            if obj.type == 'in_invoice':
                so = self.get_related_sale_order(cr, uid, obj.origin)
                if so:
                    res[obj.id] = so.partner_id.name
        return res

    def _get_other_partner_id(self, cr, uid, ids, fields, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            res[obj.id] = False
            if obj.type in ['in_invoice', 'in_refund']:
                so = self.get_related_sale_order(cr, uid, obj.origin)
                if so:
                    res[obj.id] = so.partner_id.id
        return res

    def get_in_base_currency(self, cr, uid, price, obj, context):
        base_currency_id = obj.company_id.currency_id.id
        invoice_currency_id = obj.currency_id.id
        if base_currency_id != invoice_currency_id:
            currency = self.pool.get('res.currency')
            context['date'] = obj.date_invoice
            price = currency.compute(cr, uid, invoice_currency_id, base_currency_id, price, round=False, context=context)
        return price

    def _get_sale_price(self, cr, uid, ids, field, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            price = obj.amount_total
            res[obj.id] = self.get_in_base_currency(cr, uid, price, obj, context)
        return res

    def _get_cost_price(self, cr, uid, ids, field, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            cost_inv_ids = self.get_other_invoices(cr, uid, [obj.id], context)
            price = 0
            for x in self.browse(cr, uid, cost_inv_ids):
                price += self.get_in_base_currency(cr, uid, x.amount_total, x, context)
            res[obj.id] = price
        return res

    def _get_margin(self, cr, uid, ids, field, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            if obj.sale_price > 0:
                res[obj.id] = 100 - (obj.cost_price / obj.sale_price * 100)
            else:
                res[obj.id] = 0
        return res

    def _update_invoice_line(self, cr, uid, ids, fields, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            if obj.invoice_line:
                vals_linea = {
                    'invoice_type': obj.type,
                    'invoice_state': obj.state,
                    'invoice_date': obj.date_invoice
                }
                list_updated = []
                for l in obj.invoice_line:
                    self.pool.get('account.invoice.line').write(cr, uid, l.id, vals_linea, context=None)
                    list_updated.append(l.id)
            res[obj.id] = list_updated
        return res

    def _get_invoice_line(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('account.invoice.line').browse(cr, uid, ids, context=context):
            result[line.invoice_id.id] = True
        return result.keys()

    _columns = {
        'other_partner': fields.function(_get_other_partner, method=True,
                                         type='char', string='Customer'),
        'other_partner_id': fields.function(_get_other_partner_id, method=True,
                                         type='many2one', relation='res.partner',
                                         string='Customer'),
        'sale_price': fields.function(_get_sale_price, method=True,
                                      type='float', string='Sale Price', store=True),
        'cost_price': fields.function(_get_cost_price, method=True,
                                      type='float', string='Cost Price', store=True),
        'margin': fields.function(_get_margin, method=True,
                                  type='float', string='Margin', store=True,
                                  group_operator='avg'),
        'update_invoice_line': fields.function(_update_invoice_line,
                string='Update', type='char',
                store={
                    'account.invoice': (lambda self, cr, uid, ids, c={}: ids, ['invoice_line', 'state',
                                                                               'date_invoice', 'type'], 10),
                    'account.invoice.line': (_get_invoice_line, ['price_unit', 'invoice_line_tax_id', 'quantity',
                                                                 'discount', 'invoice_id'], 20),})
    }

    def create(self, cr, uid, vals, context=None):
        inv_id = super(account_invoice, self).create(cr, uid, vals, context)
        if vals.get('origin', False) and vals.get('type', False) not in ('in_refund', 'out_refund'):
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'account.invoice', inv_id, 'invoice_open', cr)
        if vals.get('type', False) == 'out_invoice':
            self.generate_supplier_invoices(cr, uid, inv_id, context)
        return inv_id

    def write(self, cr, uid, ids, vals, context=None):
        if 'origin' in vals and not context.get('origin_updated', False):
            other_invoices = self.get_other_invoices(cr, uid, ids, context)
            if other_invoices:
                context.update({'origin_updated': True})
                self.write(cr, uid, other_invoices,
                           {'origin': vals['origin']}, context)
            origin = self.read(cr, uid, ids, ['origin'], context)[0]['origin']
            order = self.get_related_sale_order(cr, uid, origin, context)
            if order:
                self.pool.get('sale.order').write(cr, uid, order.id,
                                                  {'name': vals['origin']},
                                                  context)
        return super(account_invoice, self).write(cr, uid, ids, vals, context)

    def pay_invoice(self, cr, uid, inv, context=None):
        wf_service = netsvc.LocalService("workflow")
        vals = {
            'payment_expected_currency': inv.currency_id.id,
            'default_partner_id': self.pool.get('res.partner')._find_accounting_partner(inv.partner_id).id,
            'default_amount': inv.type in ('out_refund', 'in_refund') and -inv.residual or inv.residual,
            'default_reference': inv.name,
            'close_after_process': True,
            'invoice_type': inv.type,
            'invoice_id': inv.id,
            'default_type': inv.type in ('out_invoice','out_refund') and 'receipt' or 'payment',
            'type': inv.type in ('out_invoice','out_refund') and 'receipt' or 'payment'
        }
        voucher_id = self.pool.get('account.voucher').create(cr, uid, vals, context)
        wf_service.trg_validate(uid, 'account.voucher', voucher_id, 'proforma_voucher', cr)
        return True

    # go from canceled state to draft state
    def action_cancel_draft(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state':'draft'})
        wf_service = netsvc.LocalService("workflow")
        for inv_id in ids:
            wf_service.trg_delete(uid, 'account.invoice', inv_id, cr)
            wf_service.trg_create(uid, 'account.invoice', inv_id, cr)
        context = {}
        origin = self.read(cr, uid, ids, ['origin'], context)[0]['origin']
        order = self.get_related_sale_order(cr, uid, origin, context)
        if order:
            order_obj = self.pool.get('sale.order')
            order_obj.action_done(cr, uid, order.id, context)
        return True

    def action_cancel(self, cr, uid, ids, context=None):
        context = context or {}
        res = super(account_invoice, self).action_cancel(cr, uid, ids,
                                                         context)
        origin = self.read(cr, uid, ids, ['origin'], context)[0]['origin']
        order = self.get_related_sale_order(cr, uid, origin, context)
        if order:
            order_obj = self.pool.get('sale.order')
            order_obj.action_invoice_cancel(cr, uid, order.id, context)
        invoice_type = self.read(cr, uid, ids, ['type'], context)[0]['type']
        if invoice_type == 'out_invoice' and not context.get('cancel_updated',
                                                             False):
            other_invoices = self.get_other_invoices(cr, uid, ids, context)
            if other_invoices:
                context.update({'cancel_updated': True})
                self.action_cancel(cr, uid, other_invoices, context)
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context or {}
        context.update({'no_constraint': True})
        invoice_type = self.read(cr, uid, ids, ['type'], context)[0]['type']
        if invoice_type == 'out_invoice' and not context.get('unlink_updated',
                                                             False):
            other_invoices = self.get_other_invoices(cr, uid, ids, context)
            if other_invoices:
                context.update({'unlink_updated': True})
                self.unlink(cr, uid, other_invoices, context)
        return super(account_invoice, self).unlink(cr, uid, ids, context)

    def get_other_invoices(self, cr, uid, ids, context=None):
        origin = self.read(cr, uid, ids, ['origin'], context)[0]['origin']
        other_invoices = self.search(cr, uid, [('origin', '=', origin),
                                               ('id', 'not in', ids)],
                                     context=context)
        return other_invoices

    def get_related_sale_order(self, cr, uid, origin, context=None):
        order_obj = self.pool.get('sale.order')
        order = order_obj.search(cr, uid, [('name', '=', origin)],
                                 context=context)
        return order and order_obj.browse(cr, uid, order[0], context) or False

    def get_purchase_journal(self, cr, uid, company_id, context=None):
        journal = self.pool.get('account.journal')
        jids = journal.search(cr, uid, [('type', '=', 'purchase'),
                                        ('company_id', '=', company_id)],
                              context=context)
        return jids and jids[0] or False

    def update_lines_by_supplier(self, lines_by_supplier, supplier, d):
        if supplier in lines_by_supplier:
            lines_by_supplier[supplier].append(d)
        else:
            lines_by_supplier[supplier] = [d]
        return lines_by_supplier

    def group_invoice_lines_by_supplier(self, cr, uid, invoice, context):
        sol_obj = self.pool.get('sale.order.line')
        lines_by_supplier = {}
        for line in invoice.invoice_line:
            to_search = [('order_id.name', '=', line.origin),
                         ('product_id', '=', line.product_id.id)]
            sol_ids = sol_obj.search(cr, uid, to_search, context=context)
            if sol_ids:
                if len(sol_ids) > 1:
                    cr.execute('select order_line_id from \
                                sale_order_line_invoice_rel where \
                                invoice_id = %s', (line.id,))
                    sol_ids = cr.fetchall()[0]
                order_line = sol_obj.browse(cr, uid, sol_ids[0], context)
                supplier = sol_obj.get_supplier(order_line)
                data = {'invoice_line': line, 'sale_line': order_line}
                self.update_lines_by_supplier(lines_by_supplier, supplier, data)
        return lines_by_supplier

    def generate_supplier_invoices(self, cr, uid, inv_id, context=None):
        invoice = self.browse(cr, uid, inv_id, context)
        company_id = invoice.company_id
        journal_id = self.get_purchase_journal(cr, uid, company_id.id, context)
        vals = {
            'type': 'in_invoice',
            'state': 'draft',
            'journal_id': journal_id,
            'date_invoice': invoice.date_invoice,
            'period_id': invoice.period_id.id,
            'user_id': invoice.user_id.id,
            'company_id': company_id.id,
            'origin': invoice.origin,
            'comment': 'Generated from customer invoice ' + invoice.origin
        }

        lines_by_supplier = self.group_invoice_lines_by_supplier(cr, uid,
                                                             invoice, context)
        # repair bug the change cost currency
        sale_order = self.pool.get('sale.order')
        pl_obj = self.pool.get('product.pricelist')

        for s, lines in lines_by_supplier.items():
            data = vals.copy()
            data.update({
                'partner_id': s.id,
                'account_id': s.property_account_payable.id,
                'invoice_line': []
            })
            if lines[0]['sale_line'].cost_currency_id:
                currency_id = lines[0]['sale_line'].cost_currency_id.id
            else:
                # repair bug the change cost currency
                sale_order_obj = sale_order.browse(cr, uid, lines[0]['sale_line'].order_id.id, context=context)
                currency_id = pl_obj.browse(cr, uid, sale_order_obj.pricelist_id.id).currency_id.id
                # currency_id = company_id.currency_id.id
            data['currency_id'] = currency_id
            for l in lines:
                sl = l['sale_line']
                il = l['invoice_line']
                cost_price = self.get_cost_price(cr, uid, sl, currency_id, context)
                line_vals = {
                    'name': il.product_id.name,
                    'origin': il.invoice_id.number,
                    'product_id': il.product_id.id,
                    'account_id': il.product_id.property_account_expense.id,
                    'quantity': il.quantity,
                    # 'discount': il.discount,
                    'price_unit': cost_price,
                    'origin': invoice.origin,
                }
                data['invoice_line'].append((0, 0, line_vals))
            inv_id = self.create(cr, uid, data, context)

    def get_cost_price(self, cr, uid, line, currency_id, context=None):
        price = line.price_unit_cost
        order_currency_id = line.order_id.pricelist_id.currency_id.id
        # no remove
        if order_currency_id != currency_id:
            currency_obj = self.pool.get('res.currency')
            price = currency_obj.compute(cr, uid, order_currency_id,
                                         currency_id, price, round=False,
                                         context=context)
        return price

    def _prepare_refund(self, cr, uid, invoice, date=None, period_id=None, description=None, journal_id=None, context=None):
        res = super(account_invoice, self)._prepare_refund(cr, uid, invoice, date, period_id, description, journal_id, context)
        if invoice:
            res['origin'] = invoice.origin
            if invoice.type == 'in_invoice':
                res['supplier_invoice_number'] = invoice.number
            if invoice.type == 'out_invoice':
                res['comment'] = 'Generated from customer invoice ' + str(invoice.origin) + '. ' + (invoice.comment or '')

        return res



class account_invoice_line(Model):
    _name = 'account.invoice.line'
    _inherit = 'account.invoice.line'

    _columns = {
        'invoice_id': fields.many2one('account.invoice', 'Invoice Reference', ondelete='cascade', select=True),
        'invoice_type': fields.char('Invoice type', size=64, select=True, readonly=True, help='The parent invoice type'),
        'category_id': fields.related('product_id', 'categ_id', type='many2one', relation='product.category',
                                      string='Category', store=True,),
        'invoice_state': fields.char('Invoice Status', size=64, select=True, readonly=True,
                                     help='The parent invoice status'),
        'invoice_date': fields.date('Invoice Date', select=True, readonly=True, store=True,),
        'invoice_user': fields.related('invoice_id', 'user_id', type='many2one', relation='res.users',
                                       string='Salesperson', store=True,),
        }