# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import datetime
from openerp.osv import fields
from openerp.osv.orm import Model


class res_partner(Model):
    _name = 'res.partner'
    _inherit = 'res.partner'

    def _get_main_category(self, cr, uid, ids, field, arg, context=None):
        result = {}
        for obj in self.browse(cr, uid, ids, context):
            result[obj.id] = False
            if obj.category_id:
                result[obj.id] = obj.category_id[0].name
        return result

    def _get_reservations(self, cr, uid, ids, fields, args, context=None):
        result = {}
        order_line = self.pool.get('sale.order.line')
        for obj in self.browse(cr, uid, ids, context):
            result[obj.id] = []
            to_search = [('start_date', '>=', datetime.datetime.today())]
            if obj.customer == True:
                to_search.append(('customer_id', '=', obj.id))
            elif obj.supplier == True:
                to_search.append(('supplier_id', '=', obj.id))
            else:
                continue
            l_ids = order_line.search(cr, uid, to_search, context=context)
            result[obj.id] = l_ids
        return result

    _columns = {
        'main_category': fields.function(_get_main_category, method=True,
                                         type='char', string='Category'),
        'reservation_ids': fields.function(_get_reservations, method=True,
                                           type='many2many',
                                           relation='sale.order.line',
                                           string='Reservations')
    }

    _sql_constraints = [
        ('name_partner_unique', 'unique (name)',
         'The name of the partner must be unique !'),
    ]


class res_pax(Model):
    _name = 'res.pax'
    _inherits = {'res.partner': 'partner_id'}
    _columns = {
        'partner_id': fields.many2one('res.partner', 'Partner', required=True,
                                      ondelete="cascade", select=True),
        'pax': fields.boolean('Pax')
    }

    _defaults = {
        'pax': True
    }
