# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import datetime
from openerp.osv import fields
from openerp.osv.orm import Model


class product_category(Model):
    _name = 'product.category'
    _inherit = 'product.category'

    def name_get(self, cr, uid, ids, context=None):
        if isinstance(ids, (list, tuple)) and not len(ids):
            return []
        if isinstance(ids, (long, int)):
            ids = [ids]
        reads = self.read(cr, uid, ids, ['name'], context=context)
        return [(r['id'], r['name']) for r in reads]

    def _name_get_fnc(self, cr, uid, ids, prop, unknow_none, context=None):
        res = self.name_get(cr, uid, ids, context=context)
        return dict(res)

    _columns = {
        'complete_name': fields.function(_name_get_fnc, type="char",
                                         string='Name'),
    }


class product_type(Model):
    _name = 'product.type'
    _columns = {
        'code': fields.char('Code', size=64),
        'name': fields.char('Name', size=255),
        'categ_id': fields.many2one('product.category', 'Category')
    }


class product_product(Model):
    _name = 'product.product'
    _inherit = 'product.product'

    def _get_reservations(self, cr, uid, ids, fields, args, context=None):
        result = {}
        order_line = self.pool.get('sale.order.line')
        for obj in self.browse(cr, uid, ids, context):
            to_search = [
                ('product_id', '=', obj.id),
                ('start_date', '>=', datetime.datetime.today())
            ]
            l_ids = order_line.search(cr, uid, to_search, context=context)
            result[obj.id] = l_ids
        return result

    def _get_category(self, cr, uid, context=None):
        if context is None:
            context = {}
        name = context.get('product_type', False)
        model_data = self.pool.get('ir.model.data')
        model_data_ids = model_data.search(cr, uid, [('name', '=', name)],
                                           context=context)
        res_id = False
        if model_data_ids:
            res_id = model_data.read(cr, uid, model_data_ids[0], ['res_id'],
                                     context)['res_id']
        return res_id

    _columns = {
        'option_type_ids':
            fields.many2many('option.type', 'product_option_type_rel',
                             'product_id', 'option_type_id', 'Option Types'),
        'reservation_ids':
            fields.function(_get_reservations, method=True, type='many2many',
                            relation='sale.order.line', string='Reservations'),
        'supplier_for_email_id':
            fields.many2one('res.partner', 'Supplier'),
        # TODO: es probable que este campo no haga falta, se puede usar la
        # jerarquia de categorias de producto. En ese caso el tipo de una orden
        # de venta lo debe definir el primer nivel de la categoria.
        'product_type':
            fields.many2one('product.type', 'Type',
                            domain="[('categ_id', '=', categ_id)]")
    }

    _defaults = {
        'type': 'service',
        'categ_id': _get_category
    }

    def send_email_for_supplier(self, cr, uid, ids, context=None):
        assert len(ids) == 1, "This option should only be used for "\
                              "a single id at a time."
        imd = self.pool.get('ir.model.data')
        try:
            compose_form_id = imd.get_object_reference(cr, uid, 'mail',
                                        'email_compose_message_wizard_form')[1]
        except ValueError:
            compose_form_id = False
        ctx = dict(context)
        ctx.update({
                    # default_model: ya viene en el contexto
            'default_res_id': ids[0],
            'default_use_template': False,
            'default_composition_mode': 'comment'
        })
        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'mail.compose.message',
            'views': [(compose_form_id, 'form')],
            'view_id': compose_form_id,
            'target': 'new',
            'context': ctx,
        }

    def price_get_partner(self, cr, uid, pid, partnerinfo_id, params,
                          context=None):
        if context is None:
            context = {}
        # TODO: no debe hacer falta poner la clase en una variable, todos los
        # productos deben utilizar la misma.
        cls = 'pricelist.partnerinfo'
        to_search = [('suppinfo_id', '=', partnerinfo_id)]
        if params.get('start_date', False):
            sdate = params['start_date']
            to_search += [('start_date', '<=', sdate),
                          ('end_date', '>=', sdate)]
        if params.get('end_date', False):
            edate = params['end_date']
            to_search += [('start_date', '<=', edate),
                          ('end_date', '>=', edate)]
        categ = self.browse(cr, uid, pid, context).categ_id.name.lower()
        klas = self.pool.get('product.' + categ)
        price = getattr(klas, 'price_get_partner')(cr, uid, cls, to_search,
                                                   params, context)
        return price


class product_supplierinfo(Model):
    _name = 'product.supplierinfo'
    _inherit = 'product.supplierinfo'

    _columns = {
        'start_date': fields.date('Start date'),
        'end_date': fields.date('End date'),
        'lines': fields.integer('No. Lines')
    }

    def add_season(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context)
        dates = {
            'start_date': obj.start_date,
            'end_date': obj.end_date,
            'price': 0.0,
            'min_quantity': 0.0
        }
        to_wrt = [(0, 0, dates) for x in range(obj.lines)]
        return self.write(cr, uid, ids[0], {'pricelist_ids': to_wrt}, context)


class pricelist_partnerinfo(Model):
    _name = 'pricelist.partnerinfo'
    _inherit = 'pricelist.partnerinfo'
    _columns = {
        'start_date': fields.date('Start date'),
        'end_date': fields.date('End date'),
        'child': fields.integer('Child'),
        'sequence': fields.integer('Sequence')
    }

    _defaults = {
        'sequence': 0,
        'min_quantity': 0
    }

    _order = 'sequence'
