# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2013, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields
from openerp.osv.orm import Model


class travel_line(Model):
    _name = 'travel.line'

    def _get_categories(self, cr, uid, context=None):
        category = self.pool.get('product.category')
        category_ids = category.search(cr, uid, [('parent_id', '!=', False)],
                                       context=context)
        categories = category.read(cr, uid, category_ids, ['name'], context)
        res = [('product.' + c['name'].lower(), c['name']) for c in categories]
        return res

    def _get_product_type(self, cr, uid, ids, fields, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            res[obj.id] = obj.travel_product_id.categ_id.name.lower()
        return res

    _columns = {
        'travel_product_id': fields.reference('Product',
                                              selection=_get_categories,
                                              size=256),
        'product_type': fields.function(_get_product_type, method=True,
                                        type='char', size=128,
                                        string='Category', store=True),
        'product_description': fields.char('Description', size=255),
        'start_date': fields.date('Start Date'),
        'end_date': fields.date('End Date')
    }

    def build_attr(self, field):
        categ = field.split('_')[0]
        return {'invisible': [('product_type', '!=', categ)]}

    def build_ctx(self, base, fields):
        body = ''
        for f in fields:
            body += ", '" + f + "': " + f
        first, last = base.split("'end_date': end_date")
        return first + "'end_date': end_date" + body + last

    def get_travel_fields(self, cr, uid, context=None):
        travel_fields = {}
        categ = self.pool.get('product.category')
        categ_ids = categ.search(cr, uid, [('parent_id', '!=', False)],
                                 context=context)
        categs = categ.read(cr, uid, categ_ids, ['name'], context)
        sol = self.pool.get('sale.order.line')
        for f, v in sol.fields_get(cr, uid).iteritems():
            for c in categs:
                if f.startswith(c['name'].lower()):
                    travel_fields[f] = v
                    break
        return travel_fields

    def get_context_params(self, cr, uid, line, context=None):
        context_params = {}
        fields = self.get_travel_fields(cr, uid, context)
        for f in fields:
            if hasattr(line, f):
                value = getattr(line, f)
                if value:
                    if fields[f]['type'] == 'many2one':
                        context_params.update({f: value.id})
                    else:
                        context_params.update({f: value})
        return context_params
