# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv
import time
from lxml import etree
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta


class account_financial_report_export(osv.osv_memory):
    _name = "account.financial.report.export"

    _columns = {
        'company_id': fields.related('chart_account_id', 'company_id', type='many2one', relation='res.company',
                                     string='Company', readonly=True),
        'chart_account_id': fields.many2one('account.account', 'Chart of Account', help='Select Charts of Accounts',
                                            required=True, domain=[('parent_id', '=', False)]),
        'fiscalyear_id': fields.many2one('account.fiscalyear', 'Fiscal Year',
                                         help='Keep empty for all open fiscal year'),
        'filter_cmp': fields.selection([('date', 'Date'),
                                        ('period', 'Period'), ('not_filter', 'Not Filter')], 'Filter',
                                       required=True),
        'period_selection': fields.boolean('Period'),
        'date_selection': fields.boolean('Date'),
        'date_from': fields.date("Start Date"),
        'date_to': fields.date("End Date"),
        'target_move': fields.selection([('posted', 'All Posted Entries'),
                                         ('all', 'All Entries'),
                                         ], 'Target Moves', required=True),
        'journal_ids': fields.many2many('account.journal', string='Journals', required=True),

        'period_from': fields.many2one('account.period', 'Start Period',
                                       domain="[('fiscalyear_id', '=', fiscalyear_id)]"),
        'period_to': fields.many2one('account.period', 'End Period', domain="[('fiscalyear_id', '=', fiscalyear_id)]"),
    }

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        if context is None: context = {}
        res = super(account_financial_report_export, self).fields_view_get(cr, uid, view_id=view_id,
                                                                           view_type=view_type,
                                                                           context=context, toolbar=toolbar,
                                                                           submenu=False)
        if context.get('active_model', False) == 'account.account':
            doc = etree.XML(res['arch'])
            nodes = doc.xpath("//field[@name='chart_account_id']")
            for node in nodes:
                node.set('readonly', '1')
                node.set('help',
                         'If you print the report from Account list/form view it will not consider Charts of account')
                setup_modifiers(node, res['fields']['chart_account_id'])
            res['arch'] = etree.tostring(doc)
        return res

    def _get_account(self, cr, uid, context=None):
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        accounts = self.pool.get('account.account').search(cr, uid, [('parent_id', '=', False),
                                                                     ('company_id', '=', user.company_id.id)], limit=1)
        return accounts and accounts[0] or False

    def _get_date_from(self, cr, uid, context=None):

        date_from = time.strftime('%Y-01-01')
        return date_from

    def _get_date_to(self, cr, uid, context=None):

        date_to = time.strftime('%Y-%m-%d')
        return date_to

    def _get_fiscalyear(self, cr, uid, context=None):
        if context is None:
            context = {}
        now = time.strftime('%Y-%m-%d')
        company_id = False
        ids = context.get('active_ids', [])
        if ids and context.get('active_model') == 'account.account':
            company_id = self.pool.get('account.account').browse(cr, uid, ids[0], context=context).company_id.id
        else:  # use current company id
            company_id = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id.id
        domain = [('company_id', '=', company_id), ('date_start', '<', now), ('date_stop', '>', now)]
        fiscalyears = self.pool.get('account.fiscalyear').search(cr, uid, domain, limit=1)
        return fiscalyears and fiscalyears[0] or False

    def _get_period(self, cr, uid, context=None):
        if context is None:
            context = {}

        return True

    def _get_all_journal(self, cr, uid, context=None):
        return self.pool.get('account.journal').search(cr, uid, [])

    _defaults = {
        'fiscalyear_id': _get_fiscalyear,
        'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid,
                                                                                                 'account.common.report',
                                                                                                 context=c),
        'journal_ids': _get_all_journal,
        'chart_account_id': _get_account,
        # 'date_from': _get_date_from,
        # 'date_to': _get_date_to,
        'target_move': 'posted',
        'filter_cmp': 'not_filter',

    }

    def get_lines(self, cr, uid, data):
        lines = []
        account_obj = self.pool.get('account.account')
        currency_obj = self.pool.get('res.currency')
        accounting_report = self.pool.get('accounting.report')
        # checking =accounting_report.check_report(self, cr, uid, ids, context=None)
        ids2 = self.pool.get('account.financial.report')._get_children_by_order(cr, uid,
                                                                                [data['form']['account_report_id'][0]],
                                                                                context=data['form']['used_context'])

        for report in self.pool.get('account.financial.report').browse(cr, uid, ids2,
                                                                       context=data['form']['used_context']):
            vals = {
                'name': report.name,
                'balance': report.balance * report.sign or 0.0,
                'type': 'report',
                'level': bool(report.style_overwrite) and report.style_overwrite or report.level,
                'account_type': report.type == 'sum' and 'view' or False,
                # used to underline the financial report balances
            }

            lines.append(vals)
            account_ids = []
            if report.display_detail == 'no_detail':
                # the rest of the loop is used to display the details of the financial report, so it's not needed here.
                continue
            if report.type == 'accounts' and report.account_ids:
                account_ids = account_obj._get_children_and_consol(cr, uid, [x.id for x in report.account_ids])
            elif report.type == 'account_type' and report.account_type_ids:
                account_ids = account_obj.search(cr, uid,
                                                 [('user_type', 'in', [x.id for x in report.account_type_ids])])
            if account_ids:
                for account in account_obj.browse(cr, uid, account_ids, context=data['form']['used_context']):
                    # if there are accounts to display, we add them to the lines with a level equals to their level in
                    # the COA + 1 (to avoid having them with a too low level that would conflicts with the level of data
                    # financial reports for Assets, liabilities...)
                    if report.display_detail == 'detail_flat' and account.type == 'view':
                        continue
                    flag = False
                    vals = {
                        'name': account.code + ' ' + account.name,
                        'balance': account.balance != 0 and account.balance * report.sign or account.balance,
                        'type': 'account',
                        'level': report.display_detail == 'detail_with_hierarchy' and min(account.level + 1, 6) or 6,
                        # account.level + 1
                        'account_type': account.type,
                    }

                    if not currency_obj.is_zero(cr, uid, account.company_id.currency_id, vals['balance']):
                        flag = True

                    if flag:
                        lines.append(vals)
        return lines

    def export_report(self, cr, uid, ids, data, context=None):
        if context is None:
            context = {}
        # res = self.pool.get('accounting.report').check_report(cr, uid, ids, context=context)
        data = {}
        data['form'] = self.read(cr, uid, ids,
                                 ['date_from',
                                  'date_to',
                                  'fiscalyear_id',
                                  'chart_account_id',
                                  'account_report_id',
                                  'target_move',
                                  'period_from',
                                  'period_to',
                                  'journal_ids'], context=context)[0]
        # accounting_report = self.pool.get('accounting.report').check_report(cr, uid, ids, context)

        if not ((data['form']['period_from'] and data['form']['period_to']) or (
                data['form']['date_from'] and data['form']['date_to']) or data['form']['fiscalyear_id']):
            raise osv.except_osv(('Error'), ('You need to define period or date ranges'))

        if data['form']['fiscalyear_id']:
            fiscal_year = self.pool.get('account.fiscalyear').browse(cr, uid, data['form']['fiscalyear_id'][0],
                                                                     context=context)
            fiscal_year_date_start = datetime.strptime(fiscal_year.date_start, '%Y-%m-%d')
            fiscal_year_date_stop = datetime.strptime(fiscal_year.date_stop, '%Y-%m-%d')
            if data['form']['period_from'] and data['form']['period_to']:
                if data['form']['period_from'][0] > data['form']['period_to'][0]:
                    raise osv.except_osv(('Error'), ('Start period should be smaller then End period'))
                else:
                    # obtener fechas de los periodos
                    data['form']['date_currency_from'] = self.pool.get('account.period').browse(cr, uid,
                                                                                                data['form'][
                                                                                                    'period_from'][
                                                                                                    0]).date_start
                    data['form']['date_currency_to'] = self.pool.get('account.period').browse(cr, uid,
                                                                                             data['form']['period_to'][
                                                                                                 0]).date_stop
                    data['form']['period_from'] = data['form']['period_from'][0]
                    data['form']['period_to'] = data['form']['period_to'][0]

            elif data['form']['date_from'] and data['form']['date_to']:
                if datetime.strptime(data['form']['date_from'],
                                     '%Y-%m-%d') < fiscal_year_date_start or datetime.strptime(data['form']['date_to'],
                                                                                               '%Y-%m-%d') > fiscal_year_date_stop:
                    raise osv.except_osv(('Error'), ('Dates are not within the defined date range for the fiscal year'))
                data['form']['date_currency_from'] = data['form']['date_from']
                data['form']['date_currency_to'] = data['form']['date_to']
            else:
                data['form']['date_from'] = fiscal_year_date_start.strftime('%Y-%m-%d')
                data['form']['date_to'] = fiscal_year_date_stop.strftime('%Y-%m-%d')
                data['form']['date_currency_from'] = data['form']['date_from'].strftime('%Y-%m-%d')
                data['form']['date_currency_to'] = data['form']['date_to'].strftime('%Y-%m-%d')

        else:
            if data['form']['date_from'] > data['form']['date_to']:
                raise osv.except_osv(('Error'), ('Start date should be smaller then End date'))

        data['form']['chart_account_id'] = data['form']['chart_account_id'][0] or False
        month_rate = self.pool.get('res.currency.month.rate').search(cr, uid,
                                                                     [('month', '>=',
                                                                       data['form']['date_currency_from'][5:7]),
                                                                      ('month', '<=',
                                                                       data['form']['date_currency_to'][5:7])],
                                                                     context=context)
        currency_month = self.pool.get('res.currency.month.rate').read(cr, uid, month_rate, context=context)
        currency_name = []
        for currency in currency_month:
            if currency['currency_id'][1] not in currency_name:
                currency_name.append(currency['currency_id'][1])

        used_context = {u'lang': u'en_US', u'state': u'posted'}
        data['form']['used_context'] = used_context
        data['form']['used_context'].update(self.read(cr, uid, ids,
                                                      ['chart_account_id',
                                                       'period_from',
                                                       'period_to',
                                                       'date_from',
                                                       'date_to',
                                                       'journal_ids',
                                                       'fiscalyear_id',
                                                       'account'], context=context)[0])
        data['form']['used_context']['chart_account_id'] = data['form']['used_context']['chart_account_id'][0] or False
        if data['form']['period_from'] and  data['form']['period_to']:
            data['form']['used_context']['period_from'] = data['form']['used_context']['period_from'][0] or False
            data['form']['used_context']['period_to'] = data['form']['used_context']['period_to'][0] or False
            data['form']['used_context']['periods'] = []
            data['form']['periods'] = []
        # balance sheet
        data['form']['account_report_id'] = [4, u'Balance Sheet']

        lines_sheet = []

        lines_sheet = self.get_lines(cr, uid, data)

        balance_sheet = {
            u"Assets": [u"4", u"0"],
            u"Current Assets": [u"5", u"1"],
            u"Chequing/Savings": [u"6", u"2"],
            u"11084 GS's float": [u"7", u"3"],
            u'11081 GS BFI USD': [u"8", u"3"],
            u'11082 GS Sabadell EUR': [u"9", u"3"],
            u'11086 GS Petty Cash CUC': [u"10", u"3"],
            u'11089 Deposit in Transit': [u"11", u"3"],
            u'11092 Card (AMF)': [u"12", u"3"],
            u'Total Chequing/Savings': [u"13", u"2"],
            u'Account Receivable': [u"14", u"2"],
            u'1101 Account Receivable': [u"15", u"3"],
            u'Viajes Principal': [u"16", u"3"],
            u'Total Accounts Receivable': [u"17", u"2"],
            u'Other Current Assets': [u"18", u"2"],
            u'1103 Loans and advances': [u"19", u"3"],
            u'1206 Loan CEIBA MTC USD': [u"20", u"3"],
            u'1207 Loan CPC USD': [u"21", u"3"],
            u'1104 Prepaids and deposits': [u"22", u"2"],
            u'Deposit Cubana de Aviacion': [u"23", u"2"],
            u'Deposit SOLWAYS': [u"24", u"2"],
            u'1104 General Customer': [u"25", u"3"],
            u'1104 Prepayments': [u"26", u"3"],
            u'1102 Prepaid Annual Fees & Tax': [u"27", u"3"],
            u'Rep Office Loan Account': [u"28", u"3"],
            u'Cubanacan\'s deposit': [u"29", u"3"],
            u'Ecotur\'s deposit': [u"30", u"3"],
            u'ETECSA\'s Deposit': [u"31", u"3"],
            u'Hotel Palma Real Deposit': [u"32", u"3"],
            u'Hotel Saratoga\'s Deposit': [u"33", u"3"],
            u'Hotel Sta Isabel´s deposit': [u"34", u"3"],
            u'onte Barreto\'s deposit rent': [u"35", u"3"],
            u'Total Other Current Assets': [u"36", u"2"],
            u'Total Current Assets': [u"37", u"1"],
            u'Other Assets': [u"38", u"2"],
            u'Rep Office - equity': [u"39", u"3"],
            u'Total Other Assetsy': [u"40", u"3"],
            u'Fixed Assets': [u"41", u"1"],
            u'Computers Equipment': [u"42", u"2"],
            u'1002 Computers Equipment Depreciation': [u"43", u"3"],
            u'1001 Computers Equipment': [u"44", u"3"],
            u'Total Computers Equipment': [u"45", u"3"],
            u'Vehicles': [u"46", u"2"],
            u'1006 Vehicle Depreciation': [u"47", u"3"],
            u'1005 Vehicles': [u"48", u"3"],
            u'Total Comunications Equipment': [u"49", u"2"],
            u'Furniture Office': [u"50", u"2"],
            u'1004 Furniture Office Depreciation': [u"51", u"3"],
            u'1003 Furniture Office': [u"52", u"3"],
            u'Total Furniture Office': [u"53", u"2"],
            u'Total Fixed Assets': [u"54", u"1"],
            u'TOTAL ASSETS': [u"55", u"0"],
            u'Liability & Equity': [u"56", u"0"],
            u'Liability': [u"57", u"1"],
            u'Current Liabilities': [u"58", u"2"],
            u'Other Current Liabilities': [u"59", u"3"],
            u'CPC USD Loan Account': [u"60", u"4"],
            u'CEIBA MTC USD loan account': [u"61", u"4"],
            u'1201 Account Payable': [u"62", u"4"],
            u'Consulting fees payable': [u"63", u"4"],
            u'Provision for cost of sales': [u"64", u"4"],
            u'1202 Deferred Income': [u"65", u"4"],
            u'Other deferred income': [u"66", u"5"],
            u'Total Deferred Income': [u"67", u"4"],
            u'Total Other Current Liabilities': [u"68", u"3"],
            u'Total Current Liabilities': [u"69", u"2"],
            u'Total Liabilities': [u"70", u"1"],
            u'Equity': [u"71", u"1"],
            u'1302 Retained Earnings': [u"72", u"2"],
            u'1303 Share Capital': [u"73", u"2"],
            u'Net Income': [u"74", u"2"],
            u'Total Equity': [u"75", u"2"],
            u'TOTAL LIABILITIES & EQUITY': [u"76", u"2"]
        }

        # profit and loss
        data['form']['account_report_id'] = [1, u'Profit and Loss']

        lines_profit_loss = []

        lines_profit_loss = self.get_lines(cr, uid, data)

        profit_loss = {
            u'Ordinary Income/Expense': [u"80", u"1"],
            u'Income': [u"81", u"2"],
            u'2002 Income for Sales': [u"82", u"3"],
            u'2004 Interest Income': [u"83", u"3"],
            u'Total Income': [u"84", u"2"],
            u'Cost of Goods Sold': [u"85", u"2"],
            u'Cost of sales': [u"86", u"3"],
            u'2103 Sales Commissions': [u"87", u"4"],
            u'2255 AMF Commissions': [u"88", u"4"],
            u'Cost of sales': [u"89", u"4"],
            u'2102 Other Cost of Sales': [u"90", u"4"],
            u'Total Cost of sales': [u"91", u"3"],
            u'Total COGS': [u"92", u"2"],
            u'Gross Profit': [u"93", u"1"],
            u'Expense': [u"94", u"2"],
            u'2202 Bank Service Charges': [u"95", u"3"],
            u'2205 Courier': [u"96", u"3"],
            u'2206 Depreciation Expense': [u"97", u"3"],
            u'2212 Insurance': [u"98", u"3"],
            u'2210 Dues and Subscriptions': [u"99", u"3"],
            u'Doubtful accounts expense': [u"100", u"3"],
            u'2214 Legal Fees': [u"101", u"3"],
            u'Office costs': [u"102", u"3"],
            u'22171 Communications': [u"103", u"4"],
            u'22172 Miscellaneous': [u"104", u"4"],
            u"22173 Office's Suplies": [u"105", u"4"],
            u'22174 Office Administration': [u"106", u"4"],
            u'22175 Promotion and Marketing': [u"107", u"4"],
            u'22176 Rent': [u"108", u"4"],
            u'Total Office costs': [u"109", u"3"],
            u'Repairs': [u"110", u"3"],
            u'Staff costs': [u"111", u"3"],
            u'Consulting': [u"112", u"4"],
            u'22240 Staff cost': [u"113", u"4"],
            u'22241 Other staff costs': [u"114", u"4"],
            u'Total Staff costs': [u"115", u"3"],
            u'Travel & Entertainment': [u"116", u"3"],
            u'22275 Travel': [u"117", u"4"],
            u'22272 Entertainment': [u"118", u"4"],
            u'22271 Automobile Expense': [u"119", u"4"],
            u'Total Travel & Entertainment': [u"120", u"3"],
            u'Total Expense': [u"121", u"2"],
            u'Net Ordinary Income': [u"122", u"1"],
            u'Other Income/Expense': [u"123", u"1"],
            u'Other Income': [u"124", u"2"],
            u'2005 Other Income': [u"125", u"3"],
            u'2006 Foreign exchange gain': [u"126", u"3"],
            u'Rep Office equity adjustment': [u"127", u"3"],
            u'Total Other Income': [u"128", u"2"],
            u'Net Other Income': [u"129", u"1"],
            u'Net Income': [u"130", u"1"],
        }

        data_id = data['form']

        datas = {'ids': [data_id['id']]}
        datas['model'] = 'accounting.legal'
        datas['form'] = data
        datas['csv'] = lines_sheet
        datas['balance_sheet'] = balance_sheet
        datas['lines_profit_loss'] = lines_profit_loss
        datas['profit_loss'] = profit_loss
        datas['currency_month'] = currency_month
        datas['currency_name'] = currency_name

        return {'type': 'ir.actions.report.xml',
                'report_name': 'balance.sheet.finantial.excel',
                'nodestroy': True,
                'datas': datas, }


account_financial_report_export()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
