# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv
from openerp.osv.orm import Model
from openerp import netsvc


class res_partner(Model):
    _name = 'res.partner'
    _inherit = 'res.partner'

    _columns = {
        'account_supplier_deposit_id': fields.property(
            'account.account',
            type='many2one',
            relation='account.account',
            string="Supplier Deposit Account",
            view_load=True,
            domain="[('type', '=', 'payable')]"),
        'payment_with_deposit': fields.boolean('Payment with deposit')
    }


class account_invoice(Model):
    _name = 'account.invoice'
    _inherit = 'account.invoice'

    def create(self, cr, uid, vals, context=None):
        inv_id = super(account_invoice, self).create(cr, uid, vals, context)
        if vals.get('origin', False) and vals.get('type', False) == 'in_invoice':
            self.pool.get('account.voucher').create_voucher_from_invoice(cr, uid, inv_id, context)
        return inv_id


class account_voucher(Model):
    _name = 'account.voucher'
    _inherit = 'account.voucher'

    def create_voucher_from_invoice(self, cr, uid, invoice_id, context=None):
        journal_obj = self.pool.get('account.journal')
        move_line_obj = self.pool.get('account.move.line')
        invoice_obj = self.pool.get('account.invoice')
        invoice = invoice_obj.browse(cr, uid, invoice_id, context)

        if not invoice.partner_id.payment_with_deposit:
            return False
        deposit_account_id = invoice.partner_id.account_supplier_deposit_id

        vals = {
            'partner_id': invoice.partner_id.id,
            'journal_id': 2,
            'type': 'payment',
            'pre_line': True,
            'account_id': journal_obj.read(cr, uid, 2, ['default_credit_account_id'], context)['default_credit_account_id'][0],
            'line_dr_ids': [],
            'line_cr_ids': []
        }

        amount_to_pay = invoice.amount_total

        # Debits
        for move_line in invoice.move_id.line_id:
            if move_line.account_id.reconcile:
                line_dr_vals = {
                    'move_line_id': move_line.id,
                    'account_id': move_line.account_id.id,
                    'type': 'dr',
                    'amount': move_line.amount_residual_currency
                }
                vals['line_dr_ids'].append((0, 0, line_dr_vals))

        # Credits
        to_search = [
            ('state', '=', 'valid'),
            ('account_id.type', '=', 'payable'),
            ('account_id', '=', deposit_account_id.id),
            ('reconcile_id', '=', False),
            ('partner_id', '=', invoice.partner_id.id)
        ]
        line_ids = move_line_obj.search(cr, uid, to_search, context=context)
        for move_line in move_line_obj.browse(cr, uid, line_ids, context):
            if amount_to_pay > 0:
                line_cr_vals = {
                    'move_line_id': move_line.id,
                    'account_id': move_line.account_id.id,
                    'type': 'cr',
                    'amount': min(amount_to_pay, abs(move_line.amount_residual_currency))
                }
                vals['line_cr_ids'].append((0, 0, line_cr_vals))
                amount_to_pay -= line_cr_vals['amount']
        if amount_to_pay > 0:
            raise osv.except_osv('Error!', 'There is not enough deposit for ' + invoice.partner_id.name + '.')

        vid = self.create(cr, uid, vals, context)
        wf_service = netsvc.LocalService("workflow")
        wf_service.trg_validate(uid, 'account.voucher', vid, 'proforma_voucher', cr)
        return True
